SystemOrganization addCategory: #GODSentimentAnalysis!

Object subclass: #SAFetcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

!SAFetcher class methodsFor: 'filter' stamp: 'damc 10/4/2014 17:38'!
getPosts: tags
	"Get relevant posts from Facebook using tags as filter"

	| posts |
	posts := SNETFacebookFetcher new search: ''.
	^(FILTSocialFilter new filter: posts byTags: tags)! !

!SAFetcher class methodsFor: 'filter' stamp: 'damc 10/4/2014 17:41'!
getPostsAndTweets: tags
	"Get relevant posts from Facebookand Twitter using tags as filter"

	| posts |
	posts := SNETFetcher new search: ''.
	^(FILTSocialFilter new filter: posts byTags: tags)! !

!SAFetcher class methodsFor: 'filter' stamp: 'damc 10/4/2014 17:39'!
getTweets: tags
	"Get relevant posts from Twitter using tags as filter"

	| posts |
	posts := SNETTwitterFetcher new search: ''.
	^(FILTSocialFilter new filter: posts byTags: tags)! !

Object subclass: #SAInterface
	instanceVariableNames: 'positiveDictionary negativeDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

SAInterface subclass: #SAConsumptionAnalysis
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

!SAInterface methodsFor: 'analysis' stamp: 'damc 10/4/2014 19:35'!
analyse: queries from: source as: format
	"Analyses queries from a given source and returns the results in the given format"

	|opinions evaluations index|
	(source = 'Facebook')
		ifTrue: [opinions := SAFetcher getPosts: queries]
		ifFalse: [
			(source = 'Twitter')
				ifTrue: [opinions := SAFetcher getTweets: queries]
				ifFalse: [opinions := SAFetcher getPostsAndTweets: queries]
		].

	evaluations := OrderedCollection new.
	index := 0.
	opinions size timesRepeat: [
		index := index + 1.
		evaluations add: (SASentimentAnalyser classify: (opinions at: index) using: positiveDictionary and: negativeDictionary)
	].
	^(SAOutputGenerator export: evaluations as: format)! !

SAInterface subclass: #SAPoliticalAnalysis
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

Object subclass: #SAOutputGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

!SAOutputGenerator class methodsFor: 'process' stamp: 'damc 10/4/2014 19:15'!
export: evaluations as: format
	"Export the given evaluations as a chart or a spreadsheet"

	| positives negatives neuters index evaluation sentimentLabel|
	positives := negatives := neuters := index := 0.
	sentimentLabel := SASentimentLabel getUniqueInstance.
	evaluations size timesRepeat: [
		index := index + 1.
		evaluation := evaluations at: index.
		(sentimentLabel isNeuter: evaluation)
			ifTrue: [neuters := neuters + 1]
			ifFalse: [
				(sentimentLabel isGood: evaluation)
					ifTrue: [positives := positives + 1]
					ifFalse: [negatives := negatives + 1]
			]
	].

	(format = 'graphic')
		"ifTrue: [^(GGBarGraph new generateGraph: (positives, negatives, neuters)) ]"
		"ifFalse: [^(SSSpreadsheetData new createSheet: (positives, negatives, neuters))]"! !

Object subclass: #SASentimentAnalyser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

!SASentimentAnalyser class methodsFor: 'process' stamp: 'damc 10/4/2014 19:20'!
classify: opinion using: positiveDictionary and: negativeDictionary
	"Classify an opinion in a SASentimentLabel (good, neuter or bad)"

	| positiveOcurrences negativeOcurrences bias sentimentLabel sentiment|
	sentimentLabel := SASentimentLabel getUniqueInstance.
	positiveOcurrences := SASentimentAnalyser countOfWordsOf: positiveDictionary in: opinion.
	negativeOcurrences := SASentimentAnalyser countOfWordsOf: negativeDictionary in: opinion.
	bias := ((positiveOcurrences - negativeOcurrences) abs)/(positiveOcurrences + negativeOcurrences) asFloat.
	(bias <= 0.1)
		ifTrue: [sentiment := sentimentLabel neuterSentiment]
		ifFalse: [
			(positiveOcurrences > negativeOcurrences)
				ifTrue: [sentiment := sentimentLabel positiveSentiment]
				ifFalse: [sentiment := sentimentLabel negativeSentiment]
		].
	^sentiment! !

!SASentimentAnalyser class methodsFor: 'process' stamp: 'damc 10/4/2014 18:34'!
countOfWordsOf: dictionary in: sentence
	"Counts the number of times each word in dictionary appears in sentence"

	| processor counter index |
	processor := PCSStatisticsCalculator new.
	counter := index := 0.
	dictionary size timesRepeat: [
		index := index + 1.
		"counter := counter + (processor count: (dictionary at: index) in: sentence)."
	].
	^counter! !

Object subclass: #SASentimentLabel
	instanceVariableNames: 'labelCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSentimentAnalysis'!

SASentimentLabel class
	instanceVariableNames: 'uniqueInstance'!

SASentimentLabel class
	instanceVariableNames: 'uniqueInstance'!

!SASentimentLabel class methodsFor: 'singleton' stamp: 'damc 10/4/2014 17:27'!
getUniqueInstance
	"Returns the unique instance of SASentimentLabel class. Implementation of the Singleton pattern"

	uniqueInstance isNil
		ifTrue: [ uniqueInstance := self init ].
	^uniqueInstance! !

!SASentimentLabel class methodsFor: 'initializer' stamp: 'damc 10/4/2014 16:46'!
init
	"Constructor of SASentimentLabel"

	| instance |
	instance := self new.
	instance labelCollection: (OrderedCollection new add: #BAD; add: #NEUTER; add: #GOOD; yourself).
	^instance! !

!SASentimentLabel methodsFor: 'verifiers' stamp: 'damc 10/4/2014 17:28'!
isBad: label
	"Returns true if the label represents a negative evaluation"

	^(labelCollection at: 1 = label)! !

!SASentimentLabel methodsFor: 'verifiers' stamp: 'damc 10/4/2014 17:28'!
isGood: label
	"Returns true if the label represents a positive evaluation"

	^(labelCollection at: 3 = label)! !

!SASentimentLabel methodsFor: 'verifiers' stamp: 'damc 10/4/2014 17:28'!
isNeuter: label
	"Returns true if the label represents a neuter evaluation"

	^(labelCollection at: 2 = label)! !

!SASentimentLabel methodsFor: 'private' stamp: 'damc 10/4/2014 16:45'!
labelCollection: collection
	"Setter for the labelCollection variable"

	labelCollection := collection.
	^labelCollection! !

!SASentimentLabel methodsFor: 'sentiment-creation' stamp: 'damc 10/4/2014 17:28'!
negativeSentiment
	"Returns a label representing a negative evaluation"

	^(labelCollection at: 1)! !

!SASentimentLabel methodsFor: 'sentiment-creation' stamp: 'damc 10/4/2014 17:28'!
neuterSentiment
	"Returns a label representing a neuter evaluation"

	^(labelCollection at: 2)! !

!SASentimentLabel methodsFor: 'sentiment-creation' stamp: 'damc 10/4/2014 17:28'!
positiveSentiment
	"Returns a label representing a positive evaluation"

	^(labelCollection at: 3)! !
